# -*- coding: utf-8 -*-
"""
Created on Thu Apr  9 10:30:31 2020

@author: estefania.heredia
"""

from datetime import datetime
import pandas as pd
import numpy as np
import functionsVar

def calcular(historialSemestre):
    """
    Created on Thu Apr  9 10:30:31 2020

    @author: estefania.heredia
    """
    startTime = datetime.now()

    #mayo sept oct feb
    data = historialSemestre.copy()
    data['subState'].replace("PF", "RP", inplace=True)
    data['subState'].replace("SA", "RP", inplace=True)

    #############
    #Numero de materias total del estudiante
    numMateriasTotal = data.groupby(['id', 'codDegree'], as_index=False).size().to_frame()
    numMateriasTotal.columns = ["materiasTotal"]

    ############
    #Obtenemos solo las materias tomadas por el estudiante c/s nota c/s creditos
    dataTomadas = data[(data['tomada'] == True)]

    #############
    #Numero de materias tomadas por estudiante
    numMateriasTomadas = dataTomadas.groupby(['id', 'codDegree'], as_index=False).size().to_frame()
    numMateriasTomadas.columns = ["materiasTomadas"]

    #############
    #Numero de materias total, tomadas y tomadas actuales por estudiante
    #con condicion si materiaTomada = 0 o materiasTomadasActual=0 se toma el valor de materiaTotal
    #var: materiasTotal, modelo: materiasTotal
    #var: materiasTomadas, modelo: materiasTomadas
    #var: materiasTomadasActual, modelo: materiasTomadasActual
    numMaterias = pd.merge(numMateriasTotal, numMateriasTomadas,
                           how='outer', on=['id', 'codDegree'])
    numMaterias = numMaterias.fillna(0)
    conditions = [(numMaterias['materiasTomadas'] == 0)]
    choices = [numMaterias['materiasTotal']]
    numMaterias['materiasTomadas'] = np.select(conditions, choices,
                                               default=numMaterias['materiasTomadas'])
    numMaterias.reset_index(inplace=True)

    #############
    #Para el calculo de la cantidad de creditos hechos
    #se considera todas las materias c/s nota con creditos!=0
    ############
    dataC = data.drop(data[(data['credMateria'] == 0)].index)

    #Suma de los creditos de todas las materias AP c/s nota y con credito
    #var: creditosTotalAP, modelo: credDone
    dataS = dataC[dataC.subState == 'AP']
    credAprob = dataS.groupby(['id', 'codDegree'], as_index=False).credMateria.sum()
    credAprob.columns = ['id', 'codDegree', 'creditosDone']



    #############
    #Para el calculo de tasas se considera materias c/s nota c/s creditos
    ## Alberto dijo tasas d la total
    ############

    ############
    #Numero de materias aprobadas por estudiante
    aprobadasTotal = data[(data.subState == 'AP')]
    numMateriasApTotal = aprobadasTotal.groupby(['id', 'codDegree'],
                                                as_index=False).size().to_frame()
    numMateriasApTotal.columns = ["numMateriasAPTotal"]

    #Tasa de asignaturas aprobadas frente a todas las materias
    #var: ratePass, modelo: ratePass
    rateApTotal = pd.merge(numMateriasApTotal, numMateriasTotal,
                           how='outer', on=['id', 'codDegree'])
    rateApTotal['ratePassFinal'] = round(rateApTotal.numMateriasAPTotal/rateApTotal.materiasTotal, 2)
    rateApTotal = rateApTotal.fillna(0)
    rateApTotal = rateApTotal.drop("numMateriasAPTotal", 1).drop("materiasTotal", 1)
    rateApTotal.reset_index(inplace=True)

    ############
    #Numero de materias reprobadas por estudiante
    failTotal = data[(data.subState == 'RP')]
    numMateriasRpTotal = failTotal.groupby(['id', 'codDegree'], as_index=False).size().to_frame()
    numMateriasRpTotal.columns = ["numMateriasRPTotal"]

    #Tasa de asignaturas reprobadas  frente a todas las materias
    #var: ratePass, modelo: ratePass
    rateRpTotal = pd.merge(numMateriasRpTotal, numMateriasTotal,
                           how='outer', on=['id', 'codDegree'])
    rateRpTotal['rateFailFinal'] = round(rateRpTotal.numMateriasRPTotal/rateRpTotal.materiasTotal, 2)
    rateRpTotal = rateRpTotal.fillna(0)
    rateRpTotal = rateRpTotal.drop("numMateriasRPTotal", 1).drop("materiasTotal", 1)
    rateRpTotal.reset_index(inplace=True)

    #Numero de materias anuladas tomadas por estudiante
    anuladasTotal = data[(data.subState == 'AN')]
    numMateriasAnTotal = anuladasTotal.groupby(['id', 'codDegree'],
                                               as_index=False).size().to_frame()
    numMateriasAnTotal.columns = ["numMateriasANTotal"]

    #Ratio de asignaturas anuladas frente a todas las materias tomadas
    #var: rateAnuladas, modelo: rateAnuladas
    rateAnTotal = pd.merge(numMateriasAnTotal, numMateriasTotal,
                           how='outer', on=['id', 'codDegree'])
    rateAnTotal['rateCancelFinal'] = round(rateAnTotal.numMateriasANTotal/rateAnTotal.materiasTotal, 2)
    rateAnTotal = rateAnTotal.fillna(0)
    rateAnTotal = rateAnTotal.drop("numMateriasANTotal", 1).drop("materiasTotal", 1)
    rateAnTotal.reset_index(inplace=True)

    ############
    #Obtenemos las materias con notaFinal!=0 y que sean AP, RP, AN no exactas
    #se descartaron las notaFinal=0 AP, AN
    dataNotas = data.drop(data[(data['notaFinal'] == 0) & (data['subState'] != 'RP')].index)

    ############
    #Numero de veces que un alumno ha llegado a 2da matricula
    #var: segMat, modelo: segMat
    #aprobSeg = dataNotas[(dataNotas['call'] == 0.9) & (dataNotas['subState'] == 'AP')]
    aprobSeg = dataNotas[(dataNotas['call'] == 0.9)]
    aprobSeg = aprobSeg.groupby(['id', 'codDegree'], as_index=False).size().to_frame()
    aprobSeg.columns = ["segMat"]

    ############
    #Numero de veces que un alumno ha llegado a 3era matricula
    #var: terMat, modelo: terMat
    #reprobSeg = dataNotas[(dataNotas['call'] == 0.9) & (dataNotas['subState'] == 'RP')]
    reprobSeg = dataNotas[(dataNotas['call'] == 0.8)]
    reprobSeg = reprobSeg.groupby(['id', 'codDegree'], as_index=False).size().to_frame()
    reprobSeg.columns = ["terMat"]

    ############
    #Calculamos las medias APRP, Pond solo de aquellas materias que tengan nota
    #y c/s creditos
    ############

    #Numero de materias aprobadas o reprobadas exactas totales con nota
    apRpTotal = dataNotas[((dataNotas.subState == 'AP') | (dataNotas.subState == 'RP'))]
    numMateriasApRpTotal = apRpTotal.groupby(['id', 'codDegree'], as_index=False).size().to_frame()
    numMateriasApRpTotal.columns = ["Count"]

    #Suma de notas de materias aprobadas y reprobadas exactas totales con nota
    sumApRpTotal = apRpTotal.groupby(['id', 'codDegree'], as_index=False).notaFinal.sum()
    sumApRpTotal['notaFinal'] = sumApRpTotal['notaFinal'].fillna(0)

    #Media de notas de materias aprobadas y reprobadas exactas totales con nota
    #var: mediaAPRP, modelo: mediaAPRP
    dataMediaApRpTotal = pd.merge(sumApRpTotal, numMateriasApRpTotal,
                                  how='outer', on=['id', 'codDegree'])
    dataMediaApRpTotal = dataMediaApRpTotal.fillna(0)
    dataMediaApRpTotal['mediaAPRPTotal'] = dataMediaApRpTotal.notaFinal / dataMediaApRpTotal.Count
    dataMediaApRpTotal['mediaAPRPTotal'] = dataMediaApRpTotal['mediaAPRPTotal'].fillna(0)
    dataMediaApRpTotal['mediaAPRPTotal'] = dataMediaApRpTotal['mediaAPRPTotal'].astype(float).round(2)
    dataMediaApRpTotal = dataMediaApRpTotal.drop('notaFinal', 1).drop("Count", 1)

    #Numero de materias aprobadas o reprobadas exactas tomadas con nota
    apRpTomadas = dataNotas[((dataNotas.subState == 'AP') | (dataNotas.subState == 'RP')) & (dataNotas['tomada'] == True)]
    numMateriasApRpTomadas = apRpTomadas.groupby(['id', 'codDegree'],
                                                 as_index=False).size().to_frame()
    numMateriasApRpTomadas.columns = ["Count"]

    #Suma de notas de materias aprobadas y reprobadas tomadas con nota
    sumApRpTomadas = apRpTomadas.groupby(['id', 'codDegree'],
                                         as_index=False).notaFinal.sum()
    sumApRpTomadas['notaFinal'] = sumApRpTomadas['notaFinal'].fillna(0)

    #Media de notas de materias aprobadas y reprobadas tomadas con nota
    #var: mediaAPRP, modelo: mediaAPRP
    dataMediaApRpTomadas = pd.merge(sumApRpTomadas, numMateriasApRpTomadas,
                                    how='outer', on=['id', 'codDegree'])
    dataMediaApRpTomadas = dataMediaApRpTomadas.fillna(0)
    dataMediaApRpTomadas['mediaAPRPTomadas'] = dataMediaApRpTomadas.notaFinal / dataMediaApRpTomadas.Count
    dataMediaApRpTomadas['mediaAPRPTomadas'] = dataMediaApRpTomadas['mediaAPRPTomadas'].fillna(0)
    dataMediaApRpTomadas['mediaAPRPTomadas'] = dataMediaApRpTomadas['mediaAPRPTomadas'].astype(float).round(2)
    dataMediaApRpTomadas = dataMediaApRpTomadas.drop('notaFinal', 1).drop("Count", 1)


    ############
    #Obtenemos las materias tomadas o no que tienen nota sean APRP no exactas con credito!=0
    data = dataNotas[(dataNotas.credMateria != 0)]
    #Empezamos a calcular la media ponderada (formula UC3M)
    data.call = round(data.call*(data.notaFinal).astype(float)*data.credMateria, 2)


    #Suma de las notas poderadas anteriormente de las materias aprobadas
    dataTotal = data[(data.subState == 'AP')]
    notPondTotal = dataTotal.groupby(['id', 'codDegree'], as_index=False).call.sum()


    #Suma de creditos aprobados de materias con nota y creditos
    credAprobNotaPondTotal = dataTotal.groupby(['id', 'codDegree'],
                                               as_index=False).credMateria.sum()

    #Dividimos nota ponderada y creditos cursados
    #var: mediaPond, modelo: mediaPond
    dataMediaPondTotal = pd.merge(notPondTotal, credAprobNotaPondTotal,
                                  how='outer', on=['id', 'codDegree'])
    dataMediaPondTotal['mediaPondTotal'] = round(dataMediaPondTotal.call / dataMediaPondTotal.credMateria, 2)
    dataMediaPondTotal = dataMediaPondTotal.drop("call", 1). drop("credMateria", 1)
    dataMediaPondTotal = dataMediaPondTotal.fillna(0)



    ##calcular Gap Sgte Semestre, ste gaps sta en anios semestres
    gapEstudiantes = pd.DataFrame(historialSemestre.id.unique(), columns=['id'])
    ##get solo los anios y semestres de cada estudiante, sin las materias
    semestres_anios = historialSemestre.groupby(['id', 'year', 'semesterNum'],
                                                as_index=False).size().to_frame()
    semestres_anios = semestres_anios.reset_index()
    semestres_anios.columns = ['id', 'year', 'semesterNum', 'count']
    semestres_anios = semestres_anios.drop('count', 1)

    gap_years_no_studied = functionsVar.calcularGapSgteSemestre(semestres_anios, gapEstudiantes)
    ##poniendo el resultado de los gaps de o estudiat a su correspondiente student
    gapEstudiantes['gapSemestres'] = np.where((gapEstudiantes['id'] == gap_years_no_studied['ids']),
                                              gap_years_no_studied['gaps'], 0)


    infoEstud = pd.merge(numMaterias, aprobSeg, how='outer', on=['id', 'codDegree'])
    infoEstud = pd.merge(infoEstud, reprobSeg, how='outer', on=['id', 'codDegree'])
    infoEstud = pd.merge(infoEstud, gapEstudiantes, how='outer', on=['id'])
    infoEstud = infoEstud.fillna(0)


    dataMediaNotas = pd.merge(dataMediaApRpTotal, dataMediaApRpTomadas, how='outer', on=['id', 'codDegree'])
    dataMediaNotas = pd.merge(dataMediaNotas, dataMediaPondTotal, how='outer', on=['id', 'codDegree'])
    dataMediaNotas = dataMediaNotas.fillna(0)


    dataCreditos = pd.merge(credAprob, rateApTotal, how='outer', on=['id', 'codDegree'])
    dataCreditos = pd.merge(dataCreditos, rateRpTotal, how='outer', on=['id', 'codDegree'])
    dataCreditos = pd.merge(dataCreditos, rateAnTotal, how='outer', on=['id', 'codDegree'])
    dataCreditos = dataCreditos.fillna(0)

    print("Termino de calcvariarbes")
    print(datetime.now() - startTime)
    return infoEstud, dataMediaNotas, dataCreditos
