# -*- coding: utf-8 -*-
"""
Created on Thu Apr 3 17:55:15 2020

@author: estefania.heredia
"""

import pandas as pd
import numpy as np
#import matplotlib.pyplot as plt
from sklearn.ensemble import RandomForestClassifier
from feature_selector import FeatureSelector


#https://towardsdatascience.com/improving-random-forest-in-python-part-1-893916666cd
#https://towardsdatascience.com/a-feature-selection-tool-for-machine-learning-in-python-b64dd23710f0
#https://towardsdatascience.com/a-feature-selection-tool-for-machine-learning-in-python-b64dd23710f0
def getFeatureImportances(feature_names, x_train, y_train):
    """
    Created on Thu Apr 23 11:21:26 2020

    @author: estefania.heredia
    """
#    # Features are in train and labels are in train_labels
    fs = FeatureSelector(data=x_train, labels=feature_names)

    fs.identify_missing(missing_threshold=0.6)
    missing_features = fs.ops['missing']

    fs.identify_collinear(correlation_threshold=0.98)
#   #fs.plot_collinear(plot_all = True)
#   # list of collinear features to remove
    collinear_features = fs.ops['collinear']
#    # dataframe of collinear features
#    fs.record_collinear.head()

    fs.identify_single_unique()
#   #fs.plot_unique()
    unique_features = fs.ops['single_unique']


    print(missing_features)
    print(collinear_features)
    print(unique_features)

    features_not_in_missing = np.setdiff1d(feature_names, missing_features)
    features_not_in_unique = np.setdiff1d(features_not_in_missing, unique_features)
    features_not_in = np.setdiff1d(features_not_in_unique, collinear_features)

    train_removed = x_train[features_not_in]

    cfr = RandomForestClassifier(n_estimators=300)
    cfr.fit(train_removed, y_train)

    print("Features sorted by their score")
    # List of tuples with variable and importance
    # Sort the feature importances by most important first
    # Print out the feature and importances
    importances = cfr.feature_importances_

    feature_importances = pd.DataFrame(importances,
                                       index=features_not_in,
                                       columns=['importance']).sort_values('importance', ascending=False)
    print(feature_importances)


    ##los indices de la importancia pero el orden dl valor de los indices es
    ##segun la el valor de importancia
#    indices = np.argsort(importances)[::-1]
    ##lista segun el tam del vector d importancia 0 1 2 ...10vars
#    x_values = list(range(len(importances)))
#    plt.figure()
#    plt.bar(x_values, importances[indices], orientation = 'vertical', color = 'r', edgecolor = 'k', linewidth = 1.2)
#    plt.xticks(x_values, x_train.columns[indices], rotation='vertical')
#    plt.ylabel('Importance'); plt.xlabel('Variable'); plt.title('Variable Importances')
#    plt.show()

    # Cumulative importances
    # shows the contribution to the overall importance of each additional variable
    # Draw line at 95% of importance retained=contabilizada
    cumulative_importances = np.cumsum(feature_importances)
#    plt.figure()
#    plt.plot(x_values, cumulative_importances, 'g-')
#    plt.hlines(y = 0.95, xmin=0, xmax=len(feature_importances), color = 'r', linestyles = 'dashed')
#    plt.xticks(x_values, x_train.columns[indices], rotation = 'vertical')
#    plt.xlabel('Variable'); plt.ylabel('Cumulative Importance'); plt.title('Cumulative Importances')
#    plt.show()

    print(cumulative_importances)

    # Find number of features for cumulative importance of 95%
    # Add 1 because Python is zero-indexed
    num_features_importance = np.where(cumulative_importances > 0.98)[0][0] + 1

    print('Number of features for 98% importance:', np.where(cumulative_importances > 0.95)[0][0] + 1)

    important_feature_names = [feature_importances.index[i] for i in range(num_features_importance)]

    print(important_feature_names)

    features = {}
    features['missing_features'] = missing_features
    features['collinear_features'] = collinear_features
    features['unique_features'] = unique_features
    features['feature_importances'] = feature_importances
    features['cumulative_importances'] = cumulative_importances
    features['important_feature_names'] = important_feature_names

    return features
