"""
Created on Thu Jan  28 11:11:34 2020

@author: estefania.heredia
"""
from datetime import datetime
import psycopg2, psycopg2.extras
import pandas as pd
import numpy as np
from sqlalchemy import create_engine
from settings import DATABASE, USER, PASSWORD, HOST

def connect_DB():
    """This module preprocess the student data from DB, normalizes and selects the columns to use

    Parameters:
    estudiantes (dataframe): Description of arg1

    Returns:
    df (dataframe): Returning value

    """
    try:
        connection = psycopg2.connect(database=DATABASE,
                                      user=USER, password=PASSWORD, host=HOST)
        return connection
    except (psycopg2.Error) as error:
        print("Error while connecting to PostgreSQL: ", error)


def download(conn):
    """This module preprocess the student data from DB, normalizes and selects the columns to use

    Parameters:
    estudiantes (dataframe): Description of arg1

    Returns:
    df (dataframe): Returning value

    """
    startTime = datetime.now()
    cur = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)

    try:
        print("Bajando datos de estudiantes")
        cur.execute("SELECT * FROM dim_estudiante ORDER BY id")
        data = cur.fetchall()
        df = pd.DataFrame(np.array(data), columns=data[0].keys())
        df['matricula'] = df['matricula'].astype(str).str.strip()
    
        print("Bajando historiales de estudiantes")
        cur.execute("SELECT * FROM dim_historial_estudiante ORDER BY id")
        data = cur.fetchall()
        df1 = pd.DataFrame(np.array(data), columns=data[0].keys())

    except (psycopg2.Error) as error:
        print ("Error while inserting to PostgreSQL: ", error)

    finally:
        if conn:
            cur.close()

    print(datetime.now() - startTime)

    return df, df1




def fill_dimEstudianteFactor(conn, estudiantes_tb):
    cursor = conn.cursor()
    sql = "UPDATE dim_estudiante SET factor_original_value = %s, factor_tipo = %s, factor_final_value = %s WHERE id = %s"

    for i in range(estudiantes_tb.shape[0]):
        try:
            tb_id = estudiantes_tb.id.values[i]
            cursor.execute(sql, (estudiantes_tb.original_val.values[i],
                                 estudiantes_tb.tipo_factor.values[i],
                                 estudiantes_tb.value_final.values[i],
                                 tb_id))
            conn.commit()
        except (psycopg2.Error) as error:
            print("query error update: ", error)
            print(tb_id)

    if conn:
        cursor.close()
    print("Finalizó actualización de tb dim_estudiante!!!!!")
    
    
def upload_PredictionsResults(results_prediction):    
    engine = create_engine('postgresql://lala_usr:0t3H@hG&@200.10.150.52:5432/lala_uc3m')
    results_prediction.to_sql('dim_estudiante_dropout', engine, if_exists='replace', index=False)
    
    print("Finalizada la subida a la base de datos!!!!!")