# -*- coding: utf-8 -*-
"""
Created on Fri Aug 30 15:48:46 2019

@author: estefania.heredia
"""
from datetime import datetime
import pandas as pd
import numpy as np

def verify_dates_select_values(records_xls):
    fp_act = fp_ant = gms = ise = 0

    df_ise = records_xls.copy()
    df_ise = df_ise.drop("FECHACREACIONGMS", 1)
    df_ise = df_ise.sort_values(['FECHACREACIONISE'], ascending=[True])
    df_ise_g = df_ise.iloc[-1, :]

    df_gms = records_xls.copy()
    df_gms = df_gms.drop("FECHACREACIONISE", 1)
    df_gms = df_gms.sort_values(['FECHACREACIONGMS'], ascending=[True])
    df_gms_g = df_gms.iloc[-1, :]

    fp_ant = df_ise_g['P_ANTERIOR'].astype(float)
    fp_act = df_ise_g['P_ACTUAL'].astype(float)
    gms = df_gms_g['GMS'].astype(float)
    ise = df_ise_g['ISE'].astype(float)

    return fp_act, fp_ant, gms, ise


##escalar los valores de gms y factor p entre 0-1,
#ise ya esta dentro de estos rangos
def calc_factor_value(factorp_ant_xls, factorp_act_xls, gms_xls, ise_xls):
    valor = 0
    original = 0
    tipo = ""

    if ise_xls != 0:
        if ise_xls == 2:
            if ((gms_xls == 0 or gms_xls > 10) and factorp_act_xls == 0 and factorp_ant_xls == 0):
                valor = 1
                original = round(ise_xls, 4)
                tipo = "ISE"
            else:
                if (gms_xls != 0 and gms_xls <= 10):
                    valor = escalar_factor_economico(gms_xls, "GMS")
                    original = gms_xls
                    tipo = "GMS"
                else:
                    if factorp_act_xls != 0:
                        valor = escalar_factor_economico(factorp_act_xls, "FACTORP")
                        original = factorp_act_xls
                        tipo = "FACTORP_ACT"
                    else:
                        if factorp_ant_xls != 0:
                            valor = escalar_factor_economico(factorp_ant_xls, "FACTORP")
                            original = factorp_ant_xls
                            tipo = "FACTORP_ANT"
                        else:
                            valor = -2
        else:
            valor = original = round(ise_xls, 4)
            tipo = "ISE"
    else:
        if (gms_xls != 0 and gms_xls <= 10):
            valor = escalar_factor_economico(gms_xls, "GMS")
            original = gms_xls
            tipo = "GMS"
        else:
            if factorp_act_xls != 0:
                valor = escalar_factor_economico(factorp_act_xls, "FACTORP")
                original = factorp_act_xls
                tipo = "FACTORP_ACT"
            else:
                if factorp_ant_xls != 0:
                    valor = escalar_factor_economico(factorp_ant_xls, "FACTORP")
                    original = factorp_ant_xls
                    tipo = "FACTORP_ANT"
                else:
                    valor = -5

    return valor, original, tipo


def escalar_factor_economico(n, tipo):
    valor = rmin = rmax = 0
    tmin = 0
    tmax = 1

    if tipo == "GMS":
        rmin = -0.6
        rmax = 10
    else:
        rmin = 0
        rmax = 40

    valor = round(((((n - rmin) / (rmax - rmin)) * (tmax - tmin)) + tmin), 4)
    return valor



def calcularFactorEcnonomico(estudiantes_tb):
    startTime = datetime.now()
    file = 'admitidos_factor.xlsx'
    df = pd.read_excel(file)
    df = df.drop("ANIO_INGRESO", 1)
    df[['COD_ESTUDIANTE']] = df['COD_ESTUDIANTE'].astype(str).str.strip()
    df[['P_ANTERIOR', 'P_ACTUAL', 'GMS', 'ISE']] = df[['P_ANTERIOR', 'P_ACTUAL', 'GMS', 'ISE']].fillna(0)
    df['FECHACREACIONISE'] = pd.to_datetime(df['FECHACREACIONISE'])
    df['FECHACREACIONGMS'] = pd.to_datetime(df['FECHACREACIONGMS'])

    df['inserted'] = False
    df['repeated'] = False
    no_inserted = []

    for i in range(estudiantes_tb.shape[0]):
        query = False
        tb_oid = estudiantes_tb.id.values[i]
        tb_matricula = estudiantes_tb.matricula.values[i]

        record_xls = df[(df['COD_ESTUDIANTE'] == tb_matricula)]
        idx = df[(df['COD_ESTUDIANTE'] == tb_matricula)].index.tolist()

        if len(record_xls) == 1:
            factorp_ant_xls = record_xls['P_ANTERIOR'].values[0].astype(float)
            factorp_act_xls = record_xls['P_ACTUAL'].values[0].astype(float)
            gms_xls = record_xls['GMS'].values[0].astype(float)
            ise_xls = record_xls['ISE'].values[0].astype(float)

            valor, original, tipo = calc_factor_value(factorp_ant_xls,
                                                      factorp_act_xls,
                                                      gms_xls, ise_xls)
            query = True
        elif len(record_xls) > 1:
            df['repeated'].values[idx] = True
            factorp_ant_xls, factorp_act_xls, gms_xls, ise_xls = verify_dates_select_values(record_xls)
            valor, original, tipo = calc_factor_value(factorp_ant_xls,
                                                      factorp_act_xls,
                                                      gms_xls, ise_xls)
            query = True
        else:
            no_inserted.append(tb_oid)

        if query:
            estudiantes_tb.factor_original_value.values[i] = original
            estudiantes_tb.factor_tipo.values[i] = tipo
            estudiantes_tb.factor_final_value.values[i] = valor
    
    print("Termino de calcular el factor economico")
    print(datetime.now() - startTime)

    return df, no_inserted, estudiantes_tb



def calcularGapSgteSemestre(historialForGap, estudiantesId):
    """This module preprocess the student data from DB, normalizes and selects the columns to use

    Parameters:
    data_estudiantes (dataframe): Description of arg1
    historial (dataframe): Description of arg1

    Returns:
    df_estudiantes (dataframe): Returning value
    df (dataframe): Returning value

    """
    gaps = pd.DataFrame(columns=['ids', 'gaps'])

    for i in range(len(estudiantesId)):
        historialAcad = historialForGap[(historialForGap['id'] == estudiantesId['id'].values[i])]

        termino_act_num = termino_sgte_num = gap_t = 0
        anio_act = anio_sgte = 0

        for pos_act in range(len(historialAcad)):
            pos_sgte = pos_act + 1
            gap = 0

            if (pos_sgte != len(historialAcad)):
                anio_act = historialAcad['year'].values[pos_act]
                anio_sgte = historialAcad['year'].values[pos_sgte]

                termino_act_num = (historialAcad['semesterNum'].values[pos_act]).astype(int)
                termino_sgte_num = (historialAcad['semesterNum'].values[pos_sgte]).astype(int)

                if (anio_act == anio_sgte):
                    gap = 0
                else:
                    if (termino_act_num == termino_sgte_num):
                        gap = anio_sgte - anio_act - 0.5
                    elif(termino_act_num == 1 and termino_sgte_num in (2, 3)):
                        gap = anio_sgte - anio_act
                    else:
                        gap = anio_sgte - anio_act - 1

            gap_t = gap_t + gap

        row_df = pd.DataFrame([[estudiantesId['id'].values[i], gap_t]], columns=['ids', 'gaps'])
        gaps = pd.concat([gaps, row_df])

    gaps = gaps.reset_index(drop=True)

    print("Termino calcular gaps entre semestres")
    return gaps


def unifyCareers(data):
    """This module preprocess the student data from DB, normalizes and selects the columns to use

    Parameters:
    data_estudiantes (dataframe): Description of arg1
    historial (dataframe): Description of arg1

    Returns:
    df_estudiantes (dataframe): Returning value
    df (dataframe): Returning value

    """
    file = 'cod_carreras_padre_hijos.xlsx'
    carrerasPH = pd.read_excel(file)
    carrerasPH = carrerasPH.dropna().drop("IDNOMENCLATURATIT", 1)
    column_name = "codigocarreraactual"

    for i in range(len(carrerasPH['CODCARRERAPADRE'])):
        string = carrerasPH['CODCARRERAHIJOS'].values[i].strip()
        array = string.split(',')
        data[column_name] = np.where(((data[column_name].isin(array))),
                                     carrerasPH['CODCARRERAPADRE'].values[i],
                                     data[column_name])

    print("Termino de unificar las carreras")
    return data
