"""
Created on Thu Apr  9 01:13:01 2020

@author: estefania.heredia
"""

# -*- coding: utf-8 -*-
from datetime import datetime
import numpy as np
import pandas as pd
import functionsVar
from settings import ANIO_LIMIT_MATRICULA, ANIO_ACTUAL, SEMESTRE_ACTUAL
from settings import RETENTION_VAL, DROPOUT_VAL
from settings import COL_NAMES_HISTORIAL_DF, COL_NAMES_ESTUDIANTES_DF

def historialEstudiante(data_estudiantes, historial):
    """This module preprocess the student data from DB, normalizes and selects the columns to use

    Parameters:
    data_estudiantes (dataframe): Description of arg1
    historial (dataframe): Description of arg1

    Returns:
    df_estudiantes (dataframe): Returning value
    df (dataframe): Returning value

    """
    startTime = datetime.now()

    df = historial.copy()

    df_estudiantes = data_estudiantes.copy()
    df_estudiantes['ultMat'] = 0.0
    df_estudiantes['aniosEst'] = 0.0
    df_estudiantes['dropout'] = -1

    ####Para identificar las columnas vacías que representan el siguiente alumno
    df['codigomateriatomada'] = df['codigomateriatomada'].fillna(0)
    df = df.drop("nombremateriatomada", 1).drop("tipoformacion", 1).drop("codigocarrera", 1)
    df = df.drop("nota1", 1).drop("nota2", 1).drop("mejoramiento", 1).drop("promedio", 1)
    df = df.fillna(0)

    df = df[((df.estadomateriatomada != 'IR') & (df.estadomateriatomada != 'PE'))]
    df = df[((df.estadomateriatomada != 'IA') & (df.estadomateriatomada != 'SN'))]

    #los -1 en codmateria son de estudiantes q no tienen historial
    #y los q tienen cero son pa indicar la division del grupo de materias de cada estudiante
    #colocando identificador de estudiante a su grupo de materias
    df['identificador'] = 0
    ide = 1
    for x in range(len(df)):
        if df['codigomateriatomada'].values[x] != 0 and df['codigomateriatomada'].values[x] != "-1":
            df['identificador'].values[x] = ide
        else:
            if df['codigomateriatomada'].values[x] == 0:
                ide += 1


    #elimando los registros de division y los que representan no tener historial -1
    df = df[(df['codigomateriatomada'] != 0) & (df['codigomateriatomada'] != "-1")]

    #elimando los estudiantes que no tienen historial -1
    df_estudiantes = df_estudiantes[(df_estudiantes.id.isin(df.identificador))]

    df['anio'] = df['anio'].astype(int)
    df['tomada'] = df['tomada'].astype(bool)

    #1T,2T,1B,2B -->1S
    #3T,4T,3B,4B -->2S
    #resto 3S
    df['semester_v'] = 0
    df['num_termino'] = 0

    df['semester_v'] = np.where(((df.termino == "1S") | (df.termino == "1T") | (df.termino == "1B") | (df.termino == "2T") | (df.termino == "1B")), int(1), df.semester_v)
    df['semester_v'] = np.where(((df.termino == "2S") | (df.termino == "3T") | (df.termino == "3B") | (df.termino == "4T") | (df.termino == "4B")), int(2), df.semester_v)
    df['semester_v'] = np.where(((df.semester_v != 1) & (df.semester_v != 2)), int(3), df.semester_v)

    ##solo escojo el historial de terminos ordinario 1-2, el 3 son extraord y eso no se considera
    df = df[(df['semester_v'] != 3)]

    df = df.sort_values(['identificador', 'anio', 'semester_v'], ascending=[True, True, True])

    ##solo nos quedamos con estudiantes que tengas terminos ordinario 1-2
    df_estudiantes = df_estudiantes[(df_estudiantes.id.isin(df.identificador.unique()))]
    df_estudiantes = df_estudiantes.reset_index(drop=True)


    ##unificamos los codigos de las carreras solo en el historial
    df = functionsVar.unifyCareers(df)

    ##calcular Anios Sin Estudiar o cantidad de anios semestrs han pasado dsd su ultima matricula
    last_year_studied = df.groupby('identificador').tail(1)
    last_year_studied = last_year_studied[['identificador', 'anio', 'semester_v']]
    last_year_studied['ultMat'] = 0.0
    last_year_studied['ultMat'] = np.where(((last_year_studied['anio'] == ANIO_ACTUAL) & (last_year_studied['semester_v'] == SEMESTRE_ACTUAL)), 0, last_year_studied.ultMat)
    last_year_studied['ultMat'] = np.where(((last_year_studied['semester_v'] == SEMESTRE_ACTUAL)), ANIO_ACTUAL - last_year_studied['anio'] - 0.5, last_year_studied.ultMat)
    last_year_studied['ultMat'] = np.where(((SEMESTRE_ACTUAL == 1) & (last_year_studied['semester_v'] == 2)), ANIO_ACTUAL - last_year_studied['anio'] - 1, last_year_studied.ultMat)
    last_year_studied['ultMat'] = np.where(((SEMESTRE_ACTUAL == 2) & (last_year_studied['semester_v'] == 1)), ANIO_ACTUAL - last_year_studied['anio'], last_year_studied.ultMat)
    last_year_studied.index = df_estudiantes.index
    ##poniendo el resultado de anios semestrs sin estudiar dsd su last matricula a su correspondiente student
    df_estudiantes['ultMat'] = np.where((df_estudiantes['id'] == last_year_studied['identificador']), last_year_studied['ultMat'], 0)

    ##get solo los anios y semestres de cada estudiante, sin las materias
    semestres_anios = df.groupby(['identificador', 'anio', 'semester_v'], as_index=False).size().to_frame()

    ##calcular Anios Estudiando o cantidad de anios semestrs studiando
    year_styding = semestres_anios.groupby(['identificador']).count() ##sale el total de semestres q tiene
    year_styding['aniosEst'] = 0.0
    year_styding.columns = ['num_semestres', 'aniosEst']
    year_styding['aniosEst'] = (year_styding['num_semestres'].values/2).astype(float)
    year_styding = year_styding.reset_index(drop=False)
    ##poniendo el resultado de anios semestrs estudiando a su correspondiente student
    df_estudiantes['aniosEst'] = np.where((df_estudiantes['id'] == year_styding['identificador']), year_styding['aniosEst'], 0)


    #Definiendo dropout por anios sin estudiar y
    #graduados/egresados y perdiocarrera/inactivos
    df_estudiantes['stateStudent'].fillna("", inplace=True)
    df_estudiantes['stateStudent'] = df_estudiantes['stateStudent'].astype(str)
    df_estudiantes['stateStudent'] = df_estudiantes['stateStudent'].str.strip()

    df_estudiantes['dropout'] = np.where(((df_estudiantes.stateStudent == "E") | (df_estudiantes.stateStudent == "G")), RETENTION_VAL, df_estudiantes.dropout)
    df_estudiantes['dropout'] = np.where(((df_estudiantes.stateStudent == "I") | (df_estudiantes.stateStudent == "P")), DROPOUT_VAL, df_estudiantes.dropout)
    #Dropout despues de 5 años sin matricularse
    df_estudiantes['dropout'] = np.where(((df_estudiantes['ultMat'] >= ANIO_LIMIT_MATRICULA)  & (df_estudiantes['dropout'] != RETENTION_VAL)), DROPOUT_VAL, df_estudiantes.dropout)
    print("Termino definir dropout por anios sin estudiar, graduados/egresados, perdiocarrera/inactivos")


    ##se identifica la numeracion de cada semestres estudiado  por los alumnos
    x = 0
    for i in range(len(df_estudiantes)):
        historialAcad = df[(df['identificador'] == df_estudiantes['id'].values[i])]

        carrera_df = pd.DataFrame(historialAcad.codigocarreraactual.unique(), columns=['carrera'])
        df_estudiantes['codDegree'].values[i] = carrera_df.carrera.values[0]

        num_sem = num_ord = 1
        termino_act_num = termino_sgte_num = 0
        anio_act = anio_sgte = 0

        for y in range(len(historialAcad)):
            change_per_ord = False

            pos_act = x + y
            pos_sgte = pos_act + 1

            if y+1 == len(historialAcad):
                pos_sgte = pos_act - 1

            anio_act = df['anio'].values[pos_act]
            anio_sgte = df['anio'].values[pos_sgte]

            termino_act_num = (df['semester_v'].values[pos_act]).astype(int)
            termino_sgte_num = (df['semester_v'].values[pos_sgte]).astype(int)

            if anio_act != anio_sgte or termino_act_num != termino_sgte_num:
                num_ord += 1
                change_per_ord = True
            else:
                change_per_ord = False

            num_sem = num_ord

            if change_per_ord:
                num_sem = num_ord - 1

            df['num_termino'].values[pos_act] = num_sem

        x = x + len(historialAcad)
    ##se identificaron los semestres estudiados

    df['veztomada'].replace(0, 1, inplace=True)
    df['veztomada'].replace(2, float(0.9), inplace=True)
    df['veztomada'].replace(3, float(0.8), inplace=True)
    df['veztomada'].replace(4, float(0.7), inplace=True)
    df['veztomada'].replace(5, float(0.6), inplace=True)
    df['veztomada'].replace(6, float(0.55), inplace=True)
    df['veztomada'].replace(7, float(0.55), inplace=True)

    df_estudiantes.columns = COL_NAMES_ESTUDIANTES_DF
    df.columns = COL_NAMES_HISTORIAL_DF

    print("Termino de obtener el historial de los estudiantesssss")
    print(datetime.now() - startTime)

    return df_estudiantes, df
