# -*- coding: utf-8 -*-
"""
Created on Wed Apr  8 15:09:07 2020

@author: estefania.heredia
"""
from datetime import datetime
import numpy as np
import pandas as pd
#import matplotlib.pyplot as plt
from sklearn import metrics
from sklearn.metrics import confusion_matrix
from sklearn.model_selection import cross_val_predict


def getMetrics(model, x_train, y_train):
    """
    Created on Wed Apr  8 15:09:07 2020

    @author: estefania.heredia
    """
    startTime = datetime.now()

    print("Starting cross-validation (" +  str(len(x_train)) + ' learners)')

    y_proba_class = cross_val_predict(model, x_train, y_train, cv=10)

    TN, FP, FN, TP = confusion_matrix(y_train, y_proba_class).ravel()

    accuracy = (TP + TN)/((TP + TN + FP + FN) * 1.0)

    # Results about certificate earners
    print(str(TP) + "\t" + str(FN))
    print(str(FP) + "\t" + str(TN))

    try:
        recall = round(TP / ((TP + FN)*1.0), 2)
    except:# pylint: disable=bare-except
        recall = 0

    try:
        precision = round(TP / ((TP + FP)*1.0), 2)
    except:# pylint: disable=bare-except
        precision = 0

    try:
        specificity = round(TN / ((TN + FP)*1.0), 2)
    except:# pylint: disable=bare-except
        specificity = 0

    try:
        NPV = round(TN / ((FN+TN)*1.0), 2)
    except:# pylint: disable=bare-except
        NPV = 0

    try:
        #F_score = (2 * TP)/((2 * TP + FP + FN)*1.0)
        F_score = round((2 * precision * recall) / (precision + recall), 2)
    except:# pylint: disable=bare-except
        F_score = 0

    # Compute AUC
    try:
        y = np.array(y_train)
        pred = np.array(y_proba_class)
        fpr, tpr, thresholds = metrics.roc_curve(y, pred)
        AUC = round(metrics.auc(fpr, tpr), 2)
        RMSEsk = round(np.sqrt(metrics.mean_squared_error(y, pred)), 2)
        MAE = round(metrics.mean_absolute_error(y, pred), 2)
#        plt.figure()
#        lw = 2
#        plt.plot(fpr, tpr, color='darkorange', lw=lw,
#                 label='ROC curve (area = %0.2f)' % metrics.auc(fpr, tpr))
#        plt.plot([0, 1], [0, 1], color='navy', lw=lw, linestyle='--')
#        plt.xlim([0.0, 1.0])
#        plt.ylim([0.0, 1.05])
#        plt.xlabel('False Positive Rate')
#        plt.ylabel('True Positive Rate')
#        plt.title('Receiver operating characteristic example')
#        plt.legend(loc="lower right")
#        plt.show()

    except:# pylint: disable=bare-except
        AUC = 0
        RMSEsk = 0
        MAE = 0

    metrics_df = pd.DataFrame([[accuracy, AUC, precision,
                                recall, F_score, RMSEsk, MAE,
                                specificity, NPV,
                                TP, FP, FN, TN]],
                              columns=["accuracy", "AUC", "precision",
                                       "recall", "F1", "RMSE", "MAE",
                                       "Specificity", "NPV",
                                       "TP", "FP", "FN", "TN"])
    print(metrics_df)

    print("Termino obtener metricas")
    print(datetime.now() - startTime)

    return metrics_df
