# -*- coding: utf-8 -*-
"""
Created on Sun Apr 19 11:32:18 2020

@author: estefania.heredia
"""
from datetime import datetime
import pandas as pd
import numpy as np
from sklearn.ensemble import RandomForestClassifier
from sklearn import preprocessing
from sklearn.model_selection import GridSearchCV
import features3
import metrics
import weights


def predict(dataCarrera, typeInterpreter, classNames, classConsider):
    """
    Created on Thu Apr  9 10:30:31 2020

    @author: estefania.heredia
    """
    startTime = datetime.now()

    dataPred = dataCarrera.copy()
    dataPred = dataPred.fillna(0)

    ###preparando la data de train y test
    print("preparando  train data and test data")
    dataTrain = dataPred[dataPred.dropout != -1]

    dataF = dataPred[dataPred.dropout == -1]
    dataTest = dataPred[dataPred.dropout == -1]


    for i in range(len(dataTest)):
        if dataTest['credDoneRate'].values[i] >= 0.85:
            dataTest['ratePassFinal'].values[i] = float(dataTest['ratePassFinal'].values[i])*1.2
            dataTest['mediaAPRPTomadas'].values[i] = float(dataTest['mediaAPRPTomadas'].values[i])*1.2
            dataTest['mediaAPRPTotal'].values[i] = float(dataTest['mediaAPRPTotal'].values[i])*1.2
            dataTest['rateFailFinal'].values[i] = float(dataTest['rateFailFinal'].values[i])*0.8

        elif dataTest['credDoneRate'].values[i] >= 0.8:
            dataTest['ratePassFinal'].values[i] = float(dataTest['ratePassFinal'].values[i])*1.17
            dataTest['mediaAPRPTomadas'].values[i] = float(dataTest['mediaAPRPTomadas'].values[i])*1.17
            dataTest['mediaAPRPTotal'].values[i] = float(dataTest['mediaAPRPTotal'].values[i])*1.17
            dataTest['rateFailFinal'].values[i] = float(dataTest['rateFailFinal'].values[i])*0.83

        elif dataTest['credDoneRate'].values[i] >= 0.7:
            dataTest['ratePassFinal'].values[i] = float(dataTest['ratePassFinal'].values[i])*1.1
            dataTest['mediaAPRPTomadas'].values[i] = float(dataTest['mediaAPRPTomadas'].values[i])*1.1
            dataTest['mediaAPRPTotal'].values[i] = float(dataTest['mediaAPRPTotal'].values[i])*1.1
            dataTest['rateFailFinal'].values[i] = float(dataTest['rateFailFinal'].values[i])*0.9

        elif dataTest['credDoneRate'].values[i] >= 0.5:
            dataTest['ratePassFinal'].values[i] = float(dataTest['ratePassFinal'].values[i])*1.05
            dataTest['mediaAPRPTomadas'].values[i] = float(dataTest['mediaAPRPTomadas'].values[i])*1.05
            dataTest['mediaAPRPTotal'].values[i] = float(dataTest['mediaAPRPTotal'].values[i])*1.05
            dataTest['rateFailFinal'].values[i] = float(dataTest['rateFailFinal'].values[i])*0.95


    featuares_selected = ["gapAnios", "segMat", "terMat",
                          "ratePassFinal", "rateFailFinal", "rateCancelFinal",
                          "mediaAPRPTotal", "mediaPondTotal", "economicFactor"]


    x_train_df = dataTrain[featuares_selected]
    y_train = dataTrain["dropout"].values[:].astype(int)

    x_test_df = dataTest[featuares_selected]

    print('We have {} data train with {} variables'.format(*x_train_df.shape))
    print('We have {} data test with {} variables'.format(*x_test_df.shape))
    print("termino de separa la data de train y test")

    feature_names = [i for i in x_train_df.columns]

    ##aplucando StandardScaler a c/feature mean=0 std=1
    robust_scaler = preprocessing.StandardScaler()
    x_train = robust_scaler.fit_transform(x_train_df.values)
    x_train = pd.DataFrame(x_train, index=x_train_df.index, columns=x_train_df.columns)
    x_test = robust_scaler.fit_transform(x_test_df.values)
    x_test = pd.DataFrame(x_test, index=x_test_df.index, columns=x_test_df.columns)



    print("Obteniendo los features_importance")
    try:
        features = features3.getFeatureImportances(feature_names, x_train, y_train)
        features_not_in = np.setdiff1d(feature_names, features['important_feature_names'])

        x_train_important = x_train[features['important_feature_names']]
        x_test_important = x_test[features['important_feature_names']]

    except:
        features = -1
        print("Error, no se pudo obtener los feature_impotances")
    print("Termino de obtener los features_importance")

    print("Empezo la prediccion....")
    try:
        cfr = RandomForestClassifier(n_estimators=300)
        cfr.fit(x_train_important, y_train)

        # Create the parameter grid based on the results of random search
        param_grid = {
            'max_depth': range(6, 7, 3),
            'max_features': ['auto'],
            'min_samples_leaf': range(7, 8, 3),
            'min_samples_split': range(6, 7, 3),
            'n_estimators': range(300, 500, 100),
            'n_jobs':[-1]
            }


        # Instantiate the grid search model
        grid_search = GridSearchCV(estimator=cfr, param_grid=param_grid,
                                   cv=10, n_jobs=-1, verbose=2)
        # Fit the grid search to the data
        grid_search.fit(x_train_important, y_train)

        print("\n best_estimator_  with the best_params")
        print(grid_search.best_estimator_)
        print("\n")

        #aplicando un nuevo rf basado en mejor estimador y parametros para tener las probabilidads
        cfr1 = grid_search.best_estimator_
        cfr1.fit(x_train_important, y_train)

        y_pr2 = cfr1.predict(x_test_important)
        y_pr2 = pd.DataFrame(y_pr2)

        y_pred = cfr1.predict_proba(x_test_important).astype(float)
        y_pred = pd.DataFrame(y_pred)

        ## a la data test con all info cols (dataF) concatena la prediccion
        #positiva de retencion x1, por ser clasificacion -/+
        y_pred.columns = ['x0', 'dropout']

        results_predict = pd.DataFrame(round(y_pred['dropout'], 2))
        dataF = dataF.reset_index(drop=True)
        dataF.drop("dropout", 1, inplace=True)
        dataPredicted = pd.concat([dataF, results_predict], axis=1)

        ##data usada como train al clasificador all cols x_train & y_train
        dataFin = dataTrain.copy()

        ##concatenando las matrices que tienen all info cols porque ya se termino la prediccion
        finalResult = pd.concat([dataFin, dataPredicted], axis=0, sort=False)
        finalResult = finalResult.sort_values(by='id', ascending=True)
    except:
        print("Error, no se puede predecir para esta carrera ")
        finalResult = -1

    print("Termino de predecir")

    metrics_df = metrics.getMetrics(cfr1, x_train_important, y_train)


    print("Obteniendo los features_weights")
    try:
        w = weights.getWeights(typeInterpreter, cfr1,
                               x_train_important,
                               features['important_feature_names'],
                               classNames, classConsider, 10)
        idsTrain = dataTrain[["id", "degree"]]
        idsTrain = idsTrain.reset_index(drop=True)
        wdataTrain = pd.concat([idsTrain, w], axis=1)


        w = weights.getWeights(typeInterpreter, cfr1,
                               x_test_important,
                               features['important_feature_names'],
                               classNames, classConsider, 10)
        idsTest = dataF[["id", "degree"]]
        idsTest = idsTest.reset_index(drop=True)
        wdataTest = pd.concat([idsTest, w], axis=1)

        rW = pd.concat([wdataTrain, wdataTest], axis=0, sort=False)
        rW = rW.sort_values(by='id', ascending=True)

        for i in range(len(features_not_in)):
            rW[features_not_in[i]] = 0

    except:
        print("Error, no se puede predecir para esta carrera ")
        rW = -1
    print("Terminos de obtene los pesos")

    print(datetime.now() - startTime)

    return finalResult, features, metrics_df, rW
