# -*- coding: utf-8 -*-
"""
Created on Wed Apr  1 21:07:51 2020

@author: estefania.heredia
"""

import pandas as pd
import shap
import lime #LIME package
import lime.lime_tabular #the type of LIME analysis we’ll do


# LIME has one explainer for all models
# class_consider= 0 o 1, lo negativo/positivo de la clasificacion,
#will no dropout/will dropout
def getWeights(type_interpreter, model, x_data, features_names,
               class_names, class_consider, num_features):
    """
    Created on Wed Apr  1 21:07:51 2020

    @author: estefania.heredia
    """

    result_weights_final = pd.DataFrame(columns=features_names)

    if type_interpreter == 'shap':
        explainerModel = shap.TreeExplainer(model)
        shap_values = explainerModel.shap_values(x_data)
        result_weights_final = pd.DataFrame(shap_values[class_consider],
                                            columns=features_names)

    elif type_interpreter == 'lime':
        explainer = lime.lime_tabular.LimeTabularExplainer(x_data.values,
                                                           feature_names=features_names,
                                                           class_names=class_names,
                                                           kernel_width=5)

        for row_to_show in range(len(x_data)):
            choosen_instance = x_data.iloc[row_to_show, :]
            exp = explainer.explain_instance(choosen_instance,
                                             model.predict_proba,
                                             num_features=num_features)
            result_weights_df = pd.DataFrame(exp.as_list(), columns=['namescond', 'weights'])
            result_weights_df['feature_name'] = ''


            for row_fn in range(len(features_names)):
                fn = features_names[row_fn]

                for row_rw in range(len(result_weights_df)):
                    if fn in result_weights_df['namescond'].values[row_rw]:
                        result_weights_df['feature_name'].values[row_rw] = fn


            rw = pd.DataFrame(columns=features_names)

            for row_rw in range(len(result_weights_df)):
                col = result_weights_df['feature_name'].values[row_rw]
                val = result_weights_df['weights'].values[row_rw]
                rw.at[0, col] = val

            result_weights_final = pd.concat([result_weights_final, rw], ignore_index=True)
    else:
        print('#'* 20)
        print('assign interpreter')
        print('#'* 20)


    print("Termino get_weights.py")
    return result_weights_final
